@@ -258,7 +258,7 @@ def draw_poincare(S,
                                adjustable='box')
         ax += [axis]
         # Create the Poincare sphere
-        draw_empty_sphere(axis, angle_view=angle_view)
+        draw_empty_sphere(axis, angle_view=angle_view, axis_equal=axis_equal)
 
         # Add points from Stokes
         im_scatter = draw_stokes_points(axis,
