@@ -131,21 +131,21 @@ def prepare_variables(vars,
             return vars, obj
 
 
-def expand_objects(list, length=1, copy=False):
+def expand_objects(lista, length=1, copy=False):
     """Expand a list of objects.
 
     Parameters:
-        list (list): List of py_pol objects.
+        lista (list): List of py_pol objects.
         length (int): If some variables must be expanded but all of them have length 1, this will be the length of the new variables. Default: 1.
         copy (bool): If True, the object are copied. Default: False.
 
     Returns:
-        list (list): List of expanded objects.
+        new_list (list): List of expanded objects.
     """
     # Take the sizes and shapes
     N = 1
     shape = None
-    for obj in list:
+    for obj in lista:
         # Update size
         if obj.size > 1:
             if N == 1:
@@ -164,29 +164,35 @@ def expand_objects(list, length=1, copy=False):
 
     # Expand
     if N > 1:
-        if copy:
-            obj = obj.copy()
-        obj.shape = shape
-        obj.size = N
-        for obj in list:
-            if obj.size == 1:
-                if obj._type is 'Jones_vector':
-                    new_M = np.array(
-                        [obj.M[0, 0] * np.ones(N), obj.M[1, 0] * np.ones(N)])
-                    obj.from_matrix(new_M)
-                elif obj._type is 'Jones_matrix':
+        new_list = []
+        for obj in lista:
+            if copy:
+                new_obj = obj.copy()
+            else:
+                new_obj = obj
+            if new_obj.size == 1:
+                if new_obj._type is 'Jones_vector':
+                    new_M = np.array([
+                        new_obj.M[0, 0] * np.ones(N),
+                        new_obj.M[1, 0] * np.ones(N)
+                    ])
+                    new_obj.from_matrix(new_M)
+                elif new_obj._type is 'Jones_matrix':
                     new_M = np.array([
                         [
-                            obj.M[0, 0, 0] * np.ones(N),
-                            obj.M[0, 1, 0] * np.ones(N)
+                            new_obj.M[0, 0, 0] * np.ones(N),
+                            new_obj.M[0, 1, 0] * np.ones(N)
                         ],
                         [
-                            obj.M[1, 0, 0] * np.ones(N),
-                            obj.M[1, 1, 0] * np.ones(N)
+                            new_obj.M[1, 0, 0] * np.ones(N),
+                            new_obj.M[1, 1, 0] * np.ones(N)
                         ],
                     ])
-                    obj.from_matrix(new_M)
-    return list
+                    new_obj.from_matrix(new_M)
+            new_list += [new_obj]
+    else:
+        new_list = lista
+    return new_list
 
 
 def reshape(vars, shape_like=None, shape_fun=None, shape=None, obj=None):
@@ -909,8 +915,15 @@ def azimuth_elipt_2_charac_angles(azimuth, ellipticity, out_number=True):
     Qel = which_quad(ellipticity)
     Qaz = which_quad(azimuth)
     # Adjust
-    cond1 = (Qaz == 3) + (Qaz == 4)
+    cond1 = Qaz >= 3
     delta[cond1] = delta[cond1] + np.pi
+    cond1 = Qel == 1.5
+    delta[cond1] = np.pi / 2
+    cond1 = Qel == -1.5
+    delta[cond1] = 3 * np.pi / 2
+    cond1 = (Qaz >= 3) * (Qel == 0)
+    delta[cond1] = np.pi
+
     # Make sure the output values are in the allowed limits
     alpha = put_in_limits(alpha, "alpha", out_number=out_number)
     delta = put_in_limits(delta, "delta", out_number=out_number)
@@ -943,7 +956,9 @@ def charac_angles_2_azimuth_elipt(alpha, delta):
         ellipticity (float) [-pi/4,np.pi/4]: Elipticity angle.
     """
     # Prepare variables
-    alpha, delta = prepare_variables(vars=[alpha, delta], expand=[True, True])
+    (alpha, delta), shape = prepare_variables(vars=[alpha, delta],
+                                              expand=[True, True],
+                                              give_shape=True)
     # Check that the angles belong to the correct interval. If not, fix it.
     alpha = put_in_limits(alpha, "alpha", out_number=False)
     delta = put_in_limits(delta, "delta", out_number=False)
@@ -976,6 +991,14 @@ def charac_angles_2_azimuth_elipt(alpha, delta):
     cond3 = ((Mdelta == 1.5) + (Mdelta == 2.5) +
              (Mdelta == 3.5)) * ((Qalpha == 1.5) + (Qalpha == 2.5))
     azimuth[cond1 * cond2 * cond3] = azimuth[cond1 * cond2 * cond3] + np.pi
+    cond = (Qalpha == 1.5) * (Mdelta == 1.5)
+    ellipticity[cond] = np.pi / 4
+    azimuth[cond] = np.pi / 4
+    cond = (Qalpha == 1.5) * (Mdelta == 3.5)
+    ellipticity[cond] = -np.pi / 4
+    azimuth[cond] = np.pi / 4
+    cond = (Qalpha == 1.5) * (Mdelta >= 1.5) * (Mdelta <= 3.5)
+    azimuth[cond] = 3 * np.pi / 4
 
     # if np.abs(alpha - np.pi / 4) < tol_default and (
     #         np.abs(delta - np.pi / 2) < tol_default
@@ -1002,6 +1025,9 @@ def charac_angles_2_azimuth_elipt(alpha, delta):
     # Check that the outpit values are in the correct interval
     azimuth = put_in_limits(azimuth, "azimuth")
     ellipticity = put_in_limits(ellipticity, "ellipticity")
+    # Reshape and return
+    azimuth = np.reshape(azimuth, shape)
+    ellipticity = np.reshape(ellipticity, shape)
     return azimuth, ellipticity
 
 
@@ -1089,24 +1115,40 @@ def which_quad(angle, octant=True):
     """
     q = np.zeros_like(angle)
     if octant:
-        q = q - 1.5 * (angle == -np.pi / 4)
-        q = q - 1 * (angle > -np.pi / 4) * (angle < 0)
-        q = q + 1 * (angle > 0) * (angle < np.pi / 4)
-        q = q + 1.5 * (angle == np.pi / 4)
-        q = q + 2 * (angle > np.pi / 4) * (angle < np.pi / 2)
-        q = q + 2.5 * (angle == np.pi / 2)
-        q = q + 3 * (angle > np.pi / 2) * (angle < np.pi * 3 / 4)
-        q = q + 3.5 * (angle == np.pi * 3 / 4)
-        q = q + 4 * (angle > np.pi * 3 / 4) * (angle < np.pi)
-        q = q + 4.5 * (angle == np.pi)
+        q = q - 4.5 * (angle <= -np.pi + tol_default)
+        q = q - 4 * (angle > -np.pi + tol_default) * (
+            angle < -np.pi * 3 / 4 - tol_default)
+        q = q - 3.5 * (np.abs(angle + np.pi * 3 / 4) <= tol_default)
+        q = q - 3 * (angle > -np.pi * 3 / 4 + tol_default) * (
+            angle < -np.pi / 2 - tol_default)
+        q = q - 2.5 * (np.abs(angle + np.pi / 2) <= tol_default)
+        q = q - 2 * (angle > -np.pi / 2 + tol_default) * (
+            angle < -np.pi / 4 - tol_default)
+        q = q - 1.5 * (np.abs(angle + np.pi / 4) <= tol_default)
+        q = q - 1 * (angle > -np.pi / 4 + tol_default) * (angle < -tol_default)
+        q = q + 1 * (angle > tol_default) * (angle < np.pi / 4 - tol_default)
+        q = q + 1.5 * (np.abs(angle - np.pi / 4) <= tol_default)
+        q = q + 2 * (angle > np.pi / 4 + tol_default) * (
+            angle < np.pi / 2 - tol_default)
+        q = q + 2.5 * (np.abs(angle - np.pi / 2) <= tol_default)
+        q = q + 3 * (angle > np.pi / 2 + tol_default) * (
+            angle < np.pi * 3 / 4 - tol_default)
+        q = q + 3.5 * (np.abs(angle - np.pi * 3 / 4) <= tol_default)
+        q = q + 4 * (angle > np.pi * 3 / 4 + tol_default) * (
+            angle < np.pi - tol_default)
+        q = q + 4.5 * (angle >= np.pi - tol_default)
     else:
-        q = q + 1 * (angle > 0) * (angle < np.pi / 2)
-        q = q + 1.5 * (angle == np.pi / 2)
-        q = q + 2 * (angle > np.pi / 2) * (angle < np.pi)
-        q = q + 2.5 * (angle == np.pi)
-        q = q + 3 * (angle > np.pi) * (angle < np.pi * 3 / 2)
-        q = q + 3.5 * (angle == np.pi * 3 / 2)
-        q = q + 4 * (angle > np.pi * 3 / 2) * (angle < np.pi * 2)
+        angle = angle % (2 * np.pi)
+        q = q + 1 * (angle > tol_default) * (angle < np.pi / 2 - tol_default)
+        q = q + 1.5 * (np.abs(angle - np.pi / 2) < tol_default)
+        q = q + 2 * (angle > np.pi / 2 + tol_default) * (angle <
+                                                         np.pi - tol_default)
+        q = q + 2.5 * (np.abs(angle - np.pi) < tol_default)
+        q = q + 3 * (angle > np.pi + tol_default) * (
+            angle < np.pi * 3 / 2 - tol_default)
+        q = q + 3.5 * (np.abs(angle - np.pi * 3 / 2) < tol_default)
+        q = q + 4 * (angle > np.pi * 3 / 2 + tol_default) * (
+            angle < np.pi * 2 - tol_default)
     return q
 
 
